@@ -172,6 +172,24 @@ extern std::string global_filename;
 
 extern bool global_print_keys;
 
+const int c_max_instruments = 64;
+
+struct user_midi_bus_definition
+{
+    std::string alias;
+    int instrument[16];
+};
+
+struct user_instrument_definition
+{
+    std::string instrument;
+    bool controllers_active[128];
+    std::string controllers[128];
+};
+
+extern user_midi_bus_definition   global_user_midi_bus_definitions[c_maxBuses];
+extern user_instrument_definition global_user_instrument_definitions[c_max_instruments];
+
 /* scales */
 enum c_music_scales {
 
