@@ -621,6 +621,17 @@ mainwnd::on_key_press_event(GdkEventKey* a_ev)
                 m_entry_notes->set_text( * m_mainperf->get_screen_set_notepad(m_mainperf->get_screenset()  )); 
             }
             
+            if ( a_ev->keyval == m_mainperf->m_key_start )
+            {
+                start_playing();            
+            }
+
+            if ( a_ev->keyval == m_mainperf->m_key_stop )
+            {
+                stop_playing();            
+            }
+
+
             if( m_mainperf->get_key_events()->count( a_ev->keyval) != 0 ){
                 
                 sequence_key(  (*m_mainperf->get_key_events())[a_ev->keyval] );
