@@ -41,13 +41,24 @@ midibus::midibus( int a_localclient,
     m_clock_type = e_clock_off;
     m_inputing = false;
 
+
+    char name[60];
+    if ( global_user_midi_bus_definitions[m_id].alias.length() > 0 )
+    {
+        snprintf(name,59,"(%s)",global_user_midi_bus_definitions[m_id].alias.c_str());
+    }
+    else
+    {
+        snprintf(name,59,"(%s)",a_port_name);
+    }
+    
     /* copy names */
     char tmp[60];
-    snprintf( tmp, 59, "[%d] %d:%d (%s)",
+    snprintf( tmp, 59, "[%d] %d:%d %s",
 	      m_id,
 	      m_dest_addr_client,
 	      m_dest_addr_port,
-	      a_port_name );
+	      name );
 
     m_name = tmp;
 }
