@@ -386,7 +386,13 @@ bool midifile::parse (perform * a_perf, int a_screen_set)
                                     /* Trk Done */
                                 case 0x2f:
 
-                                    //printf("%ld TRK END\n", CurrentTime);
+                                    // If delta is 0, then another event happened at the same time 
+                                    // as the track end.  the sequence class will discard the last 
+                                    // note.  This is a fix for that.   Native Seq24 file will always
+                                    // have a Delta >= 1
+                                    if ( Delta == 0 ){
+                                        CurrentTime += 1;
+                                    }
 
                                     seq->set_length (CurrentTime, false);
                                     seq->zero_markers ();
