@@ -21,10 +21,9 @@
 #include "optionsfile.h"
 #include <iostream>
 
-optionsfile::optionsfile( string a_name )
+optionsfile::optionsfile( string a_name ) :
+    configfile( a_name )
 {
-    m_name = a_name;
-    m_pos = 0;
 }
 
 optionsfile::~optionsfile( )
@@ -34,33 +33,6 @@ optionsfile::~optionsfile( )
 
 
 
-void
-optionsfile::next_data_line( ifstream *a_file)
-{
-    a_file->getline( m_line, 1024 );
-    while (( m_line[0] == '#' || m_line[0] == ' ' || m_line[0] == 0 ) &&
-           !a_file->eof() )
-    {
-        a_file->getline( m_line, 1024 );
-    }
-}
-
-void
-optionsfile::line_after( ifstream *a_file, string a_tag)
-{
-    /* run to start */
-    a_file->clear();
-    a_file->seekg( 0, ios::beg );
-
-    a_file->getline( m_line, 1024 );
-    while ( strncmp( m_line, a_tag.c_str(), a_tag.length()) != 0  &&
-            !a_file->eof() )
-    {
-        a_file->getline( m_line, 1024 );    
-    }
-    next_data_line( a_file );
-}
-
 bool
 optionsfile::parse( perform *a_perf )
 {
