@@ -105,6 +105,8 @@ perform::perform()
     m_key_screenset_up = GDK_bracketright;
     m_key_screenset_dn = GDK_bracketleft;
 
+    m_key_start  = GDK_space;
+    m_key_stop   = GDK_Escape;
     
     m_offset = 0;
     m_control_status = 0;
@@ -194,7 +196,7 @@ perform::deinit_jack( void )
 
 #ifdef JACK_SUPPORT
 
-    if ( global_with_jack_transport  && m_jack_running){
+    if ( m_jack_running){
 
          //printf ( "deinit_jack() m_jack_running[%d]\n", m_jack_running );
         
