@@ -156,7 +156,8 @@ class perform
     long m_key_screenset_up;
     long m_key_screenset_dn;
 
-    
+    long m_key_start; 
+    long m_key_stop;
 
 
     perform();
