@@ -59,7 +59,9 @@ perfroll::perfroll( perform *a_perf,
 		Gdk::POINTER_MOTION_MASK |
 		Gdk::KEY_PRESS_MASK |
 		Gdk::KEY_RELEASE_MASK |
-		Gdk::FOCUS_CHANGE_MASK	);
+		Gdk::FOCUS_CHANGE_MASK |
+		Gdk::SCROLL_MASK );
+
 
     set_size_request( 10, 10 );
 
@@ -782,6 +784,22 @@ perfroll::on_button_release_event(GdkEventButton* a_ev)
     return true;
 }
 
+bool
+perfroll::on_scroll_event( GdkEventScroll* a_ev )
+{
+	double val = m_vadjust->get_value();
+
+    if (  a_ev->direction == GDK_SCROLL_UP ){
+		val -= m_vadjust->get_step_increment();
+    }
+    if (  a_ev->direction == GDK_SCROLL_DOWN ){
+		val += m_vadjust->get_step_increment();
+    }
+
+	m_vadjust->clamp_page( val, val + m_vadjust->get_page_size());
+    return true;
+}
+
  
 
 bool
