@@ -45,7 +45,12 @@ perftime::perftime( perform *a_perf, Adjustment *a_hadjust ): DrawingArea()
 
     m_4bar_offset = 0;
 
-    m_hadjust->signal_value_changed().connect( slot( *this, &perftime::change_horz ));
+
+    m_snap = c_ppqn;
+    m_measure_length = c_ppqn * 4;
+
+    
+    m_hadjust->signal_value_changed().connect( mem_fun( *this, &perftime::change_horz ));
 
     set_double_buffered( false );
 } 
@@ -89,7 +94,13 @@ perftime::change_horz( )
     }
 }
 
-
+void
+perftime::set_guides( int a_snap, int a_measure )
+{
+    m_snap = a_snap;
+    m_measure_length = a_measure;
+    queue_draw();
+}
 
 int 
 perftime::idle_progress( )
@@ -135,24 +146,40 @@ perftime::on_expose_event(GdkEventExpose* a_e)
     
     /* draw vert lines */
     m_gc->set_foreground(m_grey);
-    for ( int i=0; i< (m_window_x * c_perf_scale_x / (c_ppqn * 4)) + 1; i++ )
+
+    long tick_offset = (m_4bar_offset * 16 * c_ppqn);
+    long first_measure = tick_offset / m_measure_length;
+
+#if 0
+
+    0   1   2   3   4   5   6
+    |   |   |   |   |   |   |
+    |    |    |    |    |    |
+    0    1    2    3    4    5
+
+#endif
+        
+    for ( int i=first_measure;
+              i<first_measure+(m_window_x * c_perf_scale_x / (m_measure_length)) + 1; i++ )
     {
+        int x_pos = ((i * m_measure_length) - tick_offset) / c_perf_scale_x;
+        
 	/* beat */
 	m_window->draw_line(m_gc,
-			   i * c_ppqn * 4 / c_perf_scale_x,
+			   x_pos,
 			   0,
-			   i * c_ppqn * 4 / c_perf_scale_x,
+			   x_pos,
 			   m_window_y );
 	
 	char bar[5];
-	sprintf( bar, "%d", (i + m_4bar_offset * 4) + 1 ); 
+	sprintf( bar, "%d", i + 1 ); 
 	
 	m_gc->set_foreground(m_black);
 
-    p_font_renderer->render_string_on_drawable(m_gc,
-                                               (i * c_ppqn * 4 / c_perf_scale_x) + 2, 
-                                               0,
-                                               m_window, bar, font::BLACK );
+        p_font_renderer->render_string_on_drawable(m_gc,
+                                                   x_pos + 2, 
+                                                   0,
+                                                   m_window, bar, font::BLACK );
     }
 
     long left = m_mainperf->get_left_tick( );
@@ -202,16 +229,22 @@ perftime::on_button_press_event(GdkEventButton* p0)
 {
     long tick = (long) p0->x;
     tick *= c_perf_scale_x;
-    tick = tick - (tick % (c_ppqn * 4));
+    //tick = tick - (tick % (c_ppqn * 4));
     tick += (m_4bar_offset * 16 * c_ppqn);
 
+    tick = tick - (tick % m_snap);
+    
     //if ( p0->button == 2 )
     //m_mainperf->set_start_tick( tick );
     if ( p0->button == 1 )
+    {
 	m_mainperf->set_left_tick( tick );
+    }
     if ( p0->button == 3 )
-	m_mainperf->set_right_tick( tick + (c_ppqn * 4) );
-
+    {
+	m_mainperf->set_right_tick( tick + m_snap );
+    }
+    
     queue_draw();
 
 
@@ -227,10 +260,10 @@ perftime::on_button_release_event(GdkEventButton* p0)
 
 
 void
-perftime::on_size_allocate(GtkAllocation* a_r )
+perftime::on_size_allocate(Gtk::Allocation &a_r )
 {
     Gtk::DrawingArea::on_size_allocate( a_r );
 
-    m_window_x = a_r->width;
-    m_window_y = a_r->height;
+    m_window_x = a_r.get_width();
+    m_window_y = a_r.get_height();
 }
