@@ -353,13 +353,6 @@ seqdata::on_button_release_event(GdkEventButton* a_p0)
 	convert_x( m_drop_x, &tick_s );
 	convert_x( m_current_x, &tick_f );
 
-
-	if ( m_drop_y < 0 ) m_drop_y = 0;
-	if ( m_drop_y > 127 ) m_drop_y = 127;
-
-	if ( m_current_y < 0 ) m_current_y = 0;
-	if ( m_current_y > 127 ) m_current_y = 127;
-
   	m_seq->change_event_data_range( tick_s, tick_f,
 					m_status,
 					m_cc,
@@ -434,11 +427,6 @@ seqdata::on_motion_notify_event(GdkEventMotion* a_p0)
 	convert_x( adj_x_min, &tick_s );
 	convert_x( adj_x_max, &tick_f );
 
-	if ( adj_y_min < 0   ) adj_y_min = 0;
-	if ( adj_y_min > 127 ) adj_y_min = 127;
-	if ( adj_y_max < 0   ) adj_y_max = 0;
-	if ( adj_y_max > 127 ) adj_y_max = 127;	
-
 	m_seq->change_event_data_range( tick_s, tick_f,
 					m_status,
 					m_cc,
@@ -461,7 +449,7 @@ seqdata::on_motion_notify_event(GdkEventMotion* a_p0)
 bool
 seqdata::on_leave_notify_event(GdkEventCrossing* p0)
 {
-    m_dragging = false;
+    // m_dragging = false;
     update_pixmap();
     queue_draw();	    
     return true;
