@@ -150,7 +150,7 @@ seqedit::seqedit( sequence *a_seq,
     m_menu_bpm = manage( new Menu() );
     m_menu_bw = manage( new Menu() );
 
-    m_menu_midich = manage( new Menu());
+    m_menu_midich = NULL;
     m_menu_midibus = NULL;
     m_menu_sequences = NULL;
 
@@ -358,9 +358,6 @@ seqedit::create_menus( void )
         
         sprintf( b, "%d", i+1 );
         
-        m_menu_midich->items().push_back(MenuElem(b, 
-                                                  SigC::bind(mem_fun(*this,&seqedit::set_midi_channel), 
-                                                       i )));
         /* length */
         m_menu_length->items().push_back(MenuElem(b, 
                                                   SigC::bind(mem_fun(*this,&seqedit::set_measures),   
@@ -623,8 +620,8 @@ seqedit::fill_top_bar( void )
     m_tooltips->set_tip( *m_button_bus, "Select Output Bus." );
 
     m_entry_bus = manage( new Entry());
-    m_entry_bus->set_max_length(26);
-    m_entry_bus->set_width_chars(26);
+    m_entry_bus->set_max_length(60);
+    m_entry_bus->set_width_chars(60);
     m_entry_bus->set_editable( false );
 
     m_hbox->pack_start( *m_button_bus , false, false );
@@ -633,7 +630,7 @@ seqedit::fill_top_bar( void )
     /* midi channel */
     m_button_channel = manage( new Button());
     m_button_channel->add( *manage( new Image(Gdk::Pixbuf::create_from_xpm_data( midi_xpm  ))));
-    m_button_channel->signal_clicked().connect(  SigC::bind<Menu *>( mem_fun( *this, &seqedit::popup_menu), m_menu_midich  ));
+    m_button_channel->signal_clicked().connect(  mem_fun( *this, &seqedit::popup_midich_menu ));
     m_tooltips->set_tip( *m_button_channel, "Select Midi channel." );
     m_entry_channel = manage( new Entry());
     m_entry_channel->set_width_chars(2);
@@ -643,9 +640,6 @@ seqedit::fill_top_bar( void )
     m_hbox->pack_start( *m_entry_channel , false, false );
 
 
-
-
-
     /* undo */
     m_button_undo = manage( new Button());
     m_button_undo->add( *manage( new Image(Gdk::Pixbuf::create_from_xpm_data( undo_xpm  ))));
@@ -794,6 +788,7 @@ seqedit::popup_menu(Menu *a_menu)
 }
 
 
+
 void
 seqedit::popup_midibus_menu( void )
 {
@@ -813,6 +808,40 @@ seqedit::popup_midibus_menu( void )
 
 }
 
+void
+seqedit::popup_midich_menu( void )
+{
+    using namespace Menu_Helpers;
+
+    m_menu_midich = manage( new Menu());
+
+    int midi_bus = m_seq->get_midi_bus();
+
+    char b[16];
+    
+    /* midi channel menu */
+    for( int i=0; i<16; i++ ){
+        
+        sprintf( b, "%d", i+1 );
+        std::string name = string(b);
+        int instrument = global_user_midi_bus_definitions[midi_bus].instrument[i]; 
+        if ( instrument >= 0 && instrument < c_maxBuses )
+        {
+            name = name + (string(" (") + 
+                           global_user_instrument_definitions[instrument].instrument + 
+                           string(")") );
+        }
+        m_menu_midich->items().push_back(MenuElem(name, 
+                                                  SigC::bind(mem_fun(*this,&seqedit::set_midi_channel), 
+                                                       i )));
+    }
+
+    m_menu_midich->popup(0,0);
+
+
+}
+
+
 
 
 void
@@ -910,6 +939,9 @@ seqedit::popup_event_menu( void )
     bool pitch_wheel = false;
     bool ccs[128]; memset( ccs, false, sizeof(bool) * 128 );
 
+    int midi_bus = m_seq->get_midi_bus();
+    int midi_ch = m_seq->get_midi_channel();
+    
     unsigned char status, cc;
     m_seq->reset_draw_marker();
     while ( m_seq->get_next_event( &status, &cc ) == true ){
@@ -965,7 +997,15 @@ seqedit::popup_event_menu( void )
         Menu *menu_cc = manage( new Menu() ); 
         
         for( int j=0; j<16; j++ ){
-            menu_cc->items().push_back( ImageMenuElem( c_controller_names[i*16+j],
+            string controller_name( c_controller_names[i*16+j] );
+            int instrument = global_user_midi_bus_definitions[midi_bus].instrument[midi_ch];  
+            if ( instrument > -1 && instrument < c_max_instruments )
+            {
+               if ( global_user_instrument_definitions[instrument].controllers_active[i*16+j] )
+                   controller_name = global_user_instrument_definitions[instrument].controllers[i*16+j];
+            }
+
+            menu_cc->items().push_back( ImageMenuElem( controller_name,
                                                        *create_menu_image( ccs[i*16+j]),
                                                        SigC::bind(mem_fun(*this,&seqedit::set_data_type), 
                                                        (unsigned char) EVENT_CONTROL_CHANGE, i*16+j)));
@@ -1087,7 +1127,15 @@ seqedit::apply_length( int a_bpm, int a_bw, int a_measures )
 long
 seqedit::get_measures( void )
 {
-  long measures = ( m_seq->get_length() / ((m_seq->get_bpm() * (c_ppqn * 4)) /  m_seq->get_bw() ));
+
+  long units = ((m_seq->get_bpm() * (c_ppqn * 4)) /  m_seq->get_bw() );
+    
+  long measures = ( m_seq->get_length() / units );
+  if (  m_seq->get_length() % units  != 0 )
+  {
+      measures++;
+  }
+  
  
   return measures;
 }
@@ -1206,9 +1254,24 @@ seqedit::set_data_type( unsigned char a_status, unsigned char a_control  )
     if ( a_status ==  EVENT_NOTE_ON )          
 	sprintf( type, "Note On" );  
     if ( a_status ==  EVENT_AFTERTOUCH )       
-	sprintf( type, "Aftertouch" );           
-    if ( a_status ==  EVENT_CONTROL_CHANGE )   
-	sprintf( type, "Control Change - %s", c_controller_names[a_control].c_str() );  
+	sprintf( type, "Aftertouch" );       
+    
+    if ( a_status ==  EVENT_CONTROL_CHANGE )
+    {
+
+        int midi_bus = m_seq->get_midi_bus();
+        int midi_ch = m_seq->get_midi_channel();
+        
+        string controller_name( c_controller_names[a_control] );
+        int instrument = global_user_midi_bus_definitions[midi_bus].instrument[midi_ch];  
+        if ( instrument > -1 && instrument < c_max_instruments )
+        {
+           if ( global_user_instrument_definitions[instrument].controllers_active[a_control] )
+               controller_name = global_user_instrument_definitions[instrument].controllers[a_control];
+        }
+	    sprintf( type, "Control Change - %s", controller_name.c_str() );  
+    }
+    
     if ( a_status ==    EVENT_PROGRAM_CHANGE )   
 	sprintf( type, "Program Change" );       
     if ( a_status ==    EVENT_CHANNEL_PRESSURE ) 
@@ -1237,11 +1300,17 @@ bool
 seqedit::timeout( void )
 {
 
+    if (m_seq->get_raise())
+    {
+        m_seq->set_raise(false);
+        raise();
+    }
+    
     if (m_seq->is_dirty_edit() ){
 
-	m_seqroll_wid->idle_redraw();
-	m_seqevent_wid->idle_redraw();
-	m_seqdata_wid->idle_redraw();
+	    m_seqroll_wid->redraw();
+	    m_seqevent_wid->redraw();
+	    m_seqdata_wid->redraw();
     }
 
     m_seqroll_wid->draw_progress_on_window();
