@@ -59,7 +59,8 @@ seqevent::seqevent(sequence *a_seq,
     m_growing   = false;
     m_adding    = false;
     m_paste     = false;
-
+    m_painting  = false;
+    
     m_status = EVENT_NOTE_ON;
 
     set_size_request( 10, c_eventarea_y );
@@ -587,162 +588,169 @@ seqevent::on_button_press_event(GdkEventButton* a_ev)
     convert_x( c_eventevent_x, &tick_w  );
 
     /* if it was a button press */
-    if ( m_status != EVENT_NOTE_OFF &&
-	 m_status != EVENT_NOTE_ON  ){
 
-	/* set values for dragging */
-	m_drop_x = m_current_x = (int) a_ev->x + m_scroll_offset_x;
+    /* set values for dragging */
+    m_drop_x = m_current_x = (int) a_ev->x + m_scroll_offset_x;
 
-	/* reset box that holds dirty redraw spot */
-	m_old.x = 0;
-	m_old.y = 0;
-	m_old.width = 0;
-	m_old.height = 0;
+    /* reset box that holds dirty redraw spot */
+    m_old.x = 0;
+    m_old.y = 0;
+    m_old.width = 0;
+    m_old.height = 0;
 
-	if ( m_paste ){
-    
-	    snap_x( &m_current_x );
-	    convert_x( m_current_x, &tick_s );
-	    m_paste = false;
-		m_seq->push_undo();
-	    m_seq->paste_selected( tick_s, 0 );
-	} else {
-		
-		/*      left mouse button     */
-		if ( a_ev->button == 1 ){ 
-			
-			/* turn x,y in to tick/note */
-			convert_x( m_drop_x, &tick_s );
-			
-			/* shift back a few ticks */
-			tick_f = tick_s + (m_zoom);
-			tick_s -= (tick_w);
-			
-			if ( tick_s < 0 ) tick_s = 0;
-			
-			numsel = m_seq->select_events( tick_s, tick_f,
-										   m_status,
-										   m_cc );
-			
-			/* if we didnt select anyhing (user clicked empty space)
-			   unselect all notes, and start selecting */
-			
-			/* none selected, start selection box */
-			if ( numsel == 0 && !m_adding ){ 
-				
-				m_seq->unselect();	    
-				m_selecting = true;
-			}
-			
-			/* add a new note if we didnt select anything */
-			else if ( numsel == 0 && m_adding ){
-				
-				snap_x( &m_drop_x );
-				/* turn x,y in to tick/note */
-				convert_x( m_drop_x, &tick_s );
-				/* add note, length = little less than snap */
-				drop_event( tick_s );
-			}
-			
-			/* if we clicked on an event, we can start moving all selected
-			   notes */
-			else if ( numsel > 0 ){
-				
-				m_moving_init = true;
-				int note;
-				
-				/* get the box that selected elements are in */
-				m_seq->get_selected_box( &tick_s, &note, 
-                                                         &tick_f, &note );
-				
-				tick_f += tick_w;
-				
-				/* convert box to X,Y values */
-				convert_t( tick_s, &x );
-				convert_t( tick_f, &w );
-				
-				/* w is actually corrids now, so we have to change */
-				w = w-x; 
+    if ( m_paste ){
 
-                                /* set the m_selected rectangle to hold the
-				   x,y,w,h of our selected events */
-				
-				m_selected.x = x;                  
-				m_selected.width=w;
-				
-				m_selected.y = (c_eventarea_y - c_eventevent_y)/2;
-				m_selected.height = c_eventevent_y;
+        snap_x( &m_current_x );
+        convert_x( m_current_x, &tick_s );
+        m_paste = false;
+        m_seq->push_undo();
+        m_seq->paste_selected( tick_s, 0 );
+
+    } else {
+
+        /*      left mouse button     */
+        if ( a_ev->button == 1 ){ 
+
+            /* turn x,y in to tick/note */
+            convert_x( m_drop_x, &tick_s );
+
+            /* shift back a few ticks */
+            tick_f = tick_s + (m_zoom);
+            tick_s -= (tick_w);
+
+            if ( tick_s < 0 ) 
+                tick_s = 0;
+
+            if ( m_adding )
+            {
+                m_painting = true;
+
+                snap_x( &m_drop_x );
+                /* turn x,y in to tick/note */
+                convert_x( m_drop_x, &tick_s );
+                /* add note, length = little less than snap */
+
+                if ( ! m_seq->select_events( tick_s, tick_f,
+                            m_status, m_cc, sequence::e_would_select ))
+                {
+                    m_seq->push_undo();
+                    drop_event( tick_s );
+                }
+
+            }
+            else /* selecting */
+            {
+                if ( ! m_seq->select_events( tick_s, tick_f,
+                            m_status, m_cc, sequence::e_is_selected ))
+                {
+                    if ( ! (a_ev->state & GDK_CONTROL_MASK) )
+                    {
+                        m_seq->unselect();
+                    }
+
+                    numsel = m_seq->select_events( tick_s, tick_f,
+                            m_status,
+                            m_cc, sequence::e_select_one );
+
+                    /* if we didnt select anyhing (user clicked empty space)
+                       unselect all notes, and start selecting */
+
+                    /* none selected, start selection box */
+                    if ( numsel == 0 )
+                    { 
+                        m_selecting = true;
+                    }
+                    else
+                    {
+                        /// needs update
+                    }
+                }
+
+                if ( m_seq->select_events( tick_s, tick_f,
+                            m_status, m_cc, sequence::e_is_selected ))
+                {
+
+                    m_moving_init = true;
+                    int note;
+
+                    /* get the box that selected elements are in */
+                    m_seq->get_selected_box( &tick_s, &note, 
+                            &tick_f, &note );
+
+                    tick_f += tick_w;
+
+                    /* convert box to X,Y values */
+                    convert_t( tick_s, &x );
+                    convert_t( tick_f, &w );
+
+                    /* w is actually corrids now, so we have to change */
+                    w = w-x; 
+
+                    /* set the m_selected rectangle to hold the
+                       x,y,w,h of our selected events */
+
+                    m_selected.x = x;                  
+                    m_selected.width=w;
+
+                    m_selected.y = (c_eventarea_y - c_eventevent_y)/2;
+                    m_selected.height = c_eventevent_y;
 
 
-                                /* save offset that we get from the snap above */
-                                int adjusted_selected_x = m_selected.x;
-                                snap_x( &adjusted_selected_x );
-                                m_move_snap_offset_x = ( m_selected.x - adjusted_selected_x);
+                    /* save offset that we get from the snap above */
+                    int adjusted_selected_x = m_selected.x;
+                    snap_x( &adjusted_selected_x );
+                    m_move_snap_offset_x = ( m_selected.x - adjusted_selected_x);
 
-                                /* align selection for drawing */
-                                snap_x( &m_selected.x );
-				snap_x( &m_current_x );
-                                snap_x( &m_drop_x );
-                                
-			} 
-			
-		} /* end if button == 1 */
-		
-		if ( a_ev->button == 3 ){ 
-			
-			set_adding( true );
-		}
-		
-	}
+                    /* align selection for drawing */
+                    snap_x( &m_selected.x );
+                    snap_x( &m_current_x );
+                    snap_x( &m_drop_x );
 
-	/* if they clicked, something changed */
-	update_pixmap();
-	draw_pixmap_on_window();  
-	
-	return true;
-    
-    } 
-	
-    if ( m_status == EVENT_NOTE_OFF ||
-		 m_status == EVENT_NOTE_ON ) {
-		
-		get_window()->set_cursor( Gdk::Cursor( Gdk::PIRATE ));
-		return true;
+                } 
+            }
+
+        } /* end if button == 1 */
+
+        if ( a_ev->button == 3 ){ 
+
+            set_adding( true );
+        }
     }
-	
-    return false;
+
+    /* if they clicked, something changed */
+    update_pixmap();
+    draw_pixmap_on_window();  
+
+    return true;
+
+
 }
 
 
 void
 seqevent::drop_event( long a_tick )
 {
-    
+
     unsigned char status = m_status;
     unsigned char d0 = m_cc;
     unsigned char d1 = 0x40;
 
     if ( m_status == EVENT_AFTERTOUCH )
-	d0 = 0;
-    
+        d0 = 0;
+
     if ( m_status == EVENT_PROGRAM_CHANGE )
-	d0 = 0; /* d0 == new patch */
+        d0 = 0; /* d0 == new patch */
 
     if ( m_status == EVENT_CHANNEL_PRESSURE )
-	d0 = 0x40; /* d0 == pressure */
+        d0 = 0x40; /* d0 == pressure */
 
     if ( m_status == EVENT_PITCH_WHEEL )
-	d0 = 0;
+        d0 = 0;
 
-    if ( m_status != EVENT_NOTE_OFF &&
-	 m_status != EVENT_NOTE_ON ) {
-
-		m_seq->push_undo();
-	m_seq->add_event( a_tick, 
-			  status,
-			  d0,
-			  d1 );
-    }
+    m_seq->add_event( a_tick, 
+            status,
+            d0,
+            d1, true );
 }
 
 
@@ -751,7 +759,7 @@ seqevent::on_button_release_event(GdkEventButton* a_ev)
 {
     long tick_s;
     long tick_f;
-  
+
     int x,w;
     int numsel;
 
@@ -760,7 +768,7 @@ seqevent::on_button_release_event(GdkEventButton* a_ev)
     m_current_x = (int) a_ev->x  + m_scroll_offset_x;;
 
     if ( m_moving )
-	snap_x( &m_current_x );
+        snap_x( &m_current_x );
 
     int delta_x = m_current_x - m_drop_x;
 
@@ -768,37 +776,37 @@ seqevent::on_button_release_event(GdkEventButton* a_ev)
 
     if ( a_ev->button == 1 ){
 
-	if ( m_selecting ){
+        if ( m_selecting ){
 
-	    x_to_w( m_drop_x, m_current_x, &x, &w );
-	    
-	    convert_x( x,   &tick_s );
-	    convert_x( x+w, &tick_f );
-	    
-	    numsel = m_seq->select_events( tick_s, tick_f,
-					   m_status,
-					   m_cc );
-	}
+            x_to_w( m_drop_x, m_current_x, &x, &w );
 
-	if ( m_moving ){
+            convert_x( x,   &tick_s );
+            convert_x( x+w, &tick_f );
+
+            numsel = m_seq->select_events( tick_s, tick_f,
+                    m_status,
+                    m_cc, sequence::e_select );
+        }
+
+        if ( m_moving ){
 
             /* adjust for snap */
             delta_x -= m_move_snap_offset_x;
-	    
-	    /* convert deltas into screen corridinates */
-	    convert_x( delta_x, &delta_tick );
-	    
-	    /* not really notes, but still moves events */
-		m_seq->push_undo();
-	    m_seq->move_selected_notes( delta_tick, 0 );
-	}
 
-	set_adding( m_adding );
+            /* convert deltas into screen corridinates */
+            convert_x( delta_x, &delta_tick );
+
+            /* not really notes, but still moves events */
+            m_seq->push_undo();
+            m_seq->move_selected_notes( delta_tick, 0 );
+        }
+
+        set_adding( m_adding );
     }
 
     if ( a_ev->button == 3 ){
 
-	set_adding( false );
+        set_adding( false );
 
     }
 
@@ -807,7 +815,10 @@ seqevent::on_button_release_event(GdkEventButton* a_ev)
     m_moving = false;
     m_growing = false;
     m_moving_init = false;
-        
+    m_painting = false;
+
+    m_seq->unpaint_all();
+
     /* if they clicked, something changed */
     update_pixmap();
     draw_pixmap_on_window(); 
@@ -820,6 +831,9 @@ seqevent::on_button_release_event(GdkEventButton* a_ev)
 bool
 seqevent::on_motion_notify_event(GdkEventMotion* a_ev)
 {
+
+    long tick = 0;
+    
     if ( m_moving_init ){
         m_moving_init = false;
         m_moving = true;
@@ -827,13 +841,23 @@ seqevent::on_motion_notify_event(GdkEventMotion* a_ev)
 
     if ( m_selecting || m_moving || m_paste  ){
 
-	m_current_x = (int) a_ev->x  + m_scroll_offset_x;;
+        m_current_x = (int) a_ev->x  + m_scroll_offset_x;;
 
-	if ( m_moving || m_paste )
-	    snap_x( &m_current_x );
- 
-	draw_selection_on_window();
+        if ( m_moving || m_paste )
+            snap_x( &m_current_x );
+
+        draw_selection_on_window();
     }
+
+
+    if ( m_painting )
+    {
+        m_current_x = (int) a_ev->x  + m_scroll_offset_x;;
+        snap_x( &m_current_x );
+        convert_x( m_current_x, &tick );
+        drop_event( tick );
+    }
+    
     return true;
 }
 
@@ -866,7 +890,6 @@ bool
 seqevent::on_focus_in_event(GdkEventFocus*)
 {
     set_flags(Gtk::HAS_FOCUS);
-    m_seq->clear_clipboard();
     return false;
 }
 
@@ -884,46 +907,53 @@ seqevent::on_key_press_event(GdkEventKey* a_p0)
 
     if ( a_p0->type == GDK_KEY_PRESS ){
 
-	if ( a_p0->keyval ==  GDK_Delete ){
+        if ( a_p0->keyval ==  GDK_Delete ){
 
-		m_seq->push_undo();
-	    m_seq->remove_selected();
-	    ret = true;
-	}
+            m_seq->push_undo();
+            m_seq->mark_selected();
+            m_seq->remove_marked();
+            ret = true;
+        }
 
-	if ( a_p0->state & GDK_CONTROL_MASK ){
+        if ( a_p0->state & GDK_CONTROL_MASK ){
 
-	    /* cut */
-	    if ( a_p0->keyval == GDK_x || a_p0->keyval == GDK_X ){
-		
-		m_seq->copy_selected();
-		m_seq->remove_selected();
+            /* cut */
+            if ( a_p0->keyval == GDK_x || a_p0->keyval == GDK_X ){
 
-		ret = true;
-	    }
-	    /* copy */
-	    if ( a_p0->keyval == GDK_c || a_p0->keyval == GDK_C ){
-		
-		m_seq->copy_selected();
-		m_seq->unselect();
-		ret = true;
-	    }
-	    /* paste */
-	    if ( a_p0->keyval == GDK_v || a_p0->keyval == GDK_V ){
-		
-		start_paste();
-		ret = true;
-	    }
-	}
+                m_seq->copy_selected();
+                m_seq->mark_selected();
+                m_seq->remove_marked();
+
+                ret = true;
+            }
+            /* copy */
+            if ( a_p0->keyval == GDK_c || a_p0->keyval == GDK_C ){
+
+                m_seq->copy_selected();
+                ret = true;
+            }
+            /* paste */
+            if ( a_p0->keyval == GDK_v || a_p0->keyval == GDK_V ){
+
+                start_paste();
+                ret = true;
+            }
+            /* Undo */
+            if ( a_p0->keyval == GDK_z || a_p0->keyval == GDK_Z ){
+
+                m_seq->pop_undo();
+                ret = true;
+            }
+         }
     }
 
     if ( ret == true ){
-        
-        redraw(); 
-        m_seqdata_wid->redraw();
+
+        redraw();
+        m_seq->set_dirty();
         return true;
     }
     else
 
-	return false;
+        return false;
 }
