@@ -39,8 +39,8 @@ seqroll::seqroll(perform *a_perf,
 
     m_black = Gdk::Color( "black" );
     m_white = Gdk::Color( "white" );
-    m_grey  = Gdk::Color( "grey" );
-    m_dk_grey = Gdk::Color( "royalblue1" );
+    m_grey  = Gdk::Color( "gray" );
+    m_dk_grey = Gdk::Color( "gray50" );
     m_red = Gdk::Color( "orange" );
 
     colormap->alloc_color( m_black );
@@ -67,13 +67,15 @@ seqroll::seqroll(perform *a_perf,
 		Gdk::KEY_RELEASE_MASK |
 		Gdk::FOCUS_CHANGE_MASK |
 		Gdk::ENTER_NOTIFY_MASK |
-		Gdk::LEAVE_NOTIFY_MASK );
+		Gdk::LEAVE_NOTIFY_MASK |
+		Gdk::SCROLL_MASK);
 
     m_selecting = false;
     m_moving    = false;
 	m_moving_init = false;
     m_growing   = false;
     m_adding    = false;
+    m_painting  = false;
     m_paste     = false;
 
     m_old_progress_x = 0;
@@ -319,7 +321,6 @@ seqroll::draw_background()
     int ticks_per_m_line =  ticks_per_measure * measures_per_line;
     int start_tick = m_scroll_offset_ticks - (m_scroll_offset_ticks % ticks_per_step );
     int end_tick = (m_window_x * m_zoom) + m_scroll_offset_ticks;
-
     
 
     //printf ( "ticks_per_step[%d] start_tick[%d] end_tick[%d]\n",
@@ -342,7 +343,7 @@ seqroll::draw_background()
             
         } else if (i % ticks_per_beat == 0 ){
             
-            m_gc->set_foreground(m_grey);
+            m_gc->set_foreground(m_dk_grey);
             m_gc->set_line_attributes( 1,
                                        Gdk::LINE_SOLID,
                                        Gdk::CAP_NOT_LAST,
@@ -353,11 +354,19 @@ seqroll::draw_background()
       
         else {
 
-            m_gc->set_foreground(m_grey);
             m_gc->set_line_attributes( 1,
-                                       Gdk::LINE_ON_OFF_DASH,
-                                       Gdk::CAP_NOT_LAST,
-                                       Gdk::JOIN_MITER );
+                        Gdk::LINE_ON_OFF_DASH,
+                        Gdk::CAP_NOT_LAST,
+                        Gdk::JOIN_MITER );
+  
+            
+            int i_snap = i - (i % m_snap);
+
+            if( i == i_snap ){
+                m_gc->set_foreground(m_dk_grey);
+            } else {
+                m_gc->set_foreground(m_grey);
+            }
             gint8 dash = 1;
             m_gc->set_dashes( 0, &dash, 1 );
         }
@@ -392,6 +401,7 @@ void
 seqroll::set_snap( int a_snap )
 {
     m_snap = a_snap;
+    reset();
 }
 
 
@@ -535,7 +545,7 @@ void seqroll::draw_events_on( Glib::RefPtr<Gdk::Drawable> a_draw )
                     
                 }
                 else {
-                    note_width = 16 / m_zoom;
+                    note_width = 8 / m_zoom;
                 }
                 
                 if ( dt == DRAW_NOTE_ON ){
@@ -862,136 +872,164 @@ seqroll::on_button_press_event(GdkEventButton* a_ev)
 	
     /* y is always snapped */
     m_current_y = m_drop_y = snapped_y;
-	
+
     /* reset box that holds dirty redraw spot */
     m_old.x = 0;
     m_old.y = 0;
     m_old.width = 0;
     m_old.height = 0;
-	
+
     if ( m_paste ){
-		
-		convert_xy( snapped_x, snapped_y, &tick_s, &note_h );
-		m_paste = false;
-		m_seq->push_undo();
-		m_seq->paste_selected( tick_s, note_h );
+
+        convert_xy( snapped_x, snapped_y, &tick_s, &note_h );
+        m_paste = false;
+        m_seq->push_undo();
+        m_seq->paste_selected( tick_s, note_h );
 
         needs_update = true;
-    
-	} else { 
-		
-		/*      left mouse button     */
-		if ( a_ev->button == 1 ){ 
-			
-			/* selection, normal x */
-			m_current_x = m_drop_x = norm_x;
-			
-			/* turn x,y in to tick/note */
-			convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
-			
-			/* set to playing */
-			numsel = m_seq->select_note_events( tick_s, note_h, 
-												tick_s, note_h );
-			
-			/* none selected, start selection box */
-			if ( numsel == 0 && !m_adding ){ 
-				
-				m_seq->unselect();	    
-				m_selecting = true;
-			}
-			
-			/* add a new note if we didnt select anything */
-			else if ( numsel == 0 && m_adding ){
-				
-				/* adding, snapped x */
-				m_current_x = m_drop_x = snapped_x;
-				convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
-				
-				/* add note, length = little less than snap */
-				m_seq->push_undo();
-				m_seq->add_note( tick_s, m_note_length - 2, note_h );
 
-                                needs_update = true;
-			}
-			
-			/* if we clicked on an event, we can start moving all selected
-			   notes */
-			else if ( numsel > 0 ){
-                            
-                            m_moving_init = true;
-                            needs_update = true;
-                            
-                                                        
-                            /* get the box that selected elements are in */
-                            m_seq->get_selected_box( &tick_s, &note_h, 
-                                                     &tick_f, &note_l );
-                            
-                            
-                            convert_tn_box_to_rect( tick_s, tick_f, note_h, note_l,
-                                                    &m_selected.x,
-                                                    &m_selected.y,
-                                                    &m_selected.width,
-                                                    &m_selected.height );
+    } else { 
 
+        /*  left mouse button     */
+        if ( a_ev->button == 1 ||
+             a_ev->button == 2 )
+        { 
+
+            /* selection, normal x */
+            m_current_x = m_drop_x = norm_x;
+
+            /* turn x,y in to tick/note */
+            convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
+
+            if ( m_adding )
+            {
+                /* start the paint job */
+                m_painting = true;
+
+                /* adding, snapped x */
+                m_current_x = m_drop_x = snapped_x;
+                convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
+
+                // test if a note is already there
+                // fake select, if so, no add
+                if ( ! m_seq->select_note_events( tick_s, note_h, 
+                                                  tick_s, note_h,
+                                                  sequence::e_would_select ))
+                {
+                
+                    /* add note, length = little less than snap */
+                    m_seq->push_undo();
+                    m_seq->add_note( tick_s, m_note_length - 2, note_h, true );
+
+                    needs_update = true;
+                }
+
+            }
+            else /* selecting */
+            {
+
+               
+                if ( !m_seq->select_note_events( tick_s, note_h,
+                                                tick_s, note_h, 
+                                                sequence::e_is_selected ))
+                {
+                    if ( ! (a_ev->state & GDK_CONTROL_MASK) )
+                    {
+                        m_seq->unselect();	    
+                    }
+ 
+
+                    /* on direct click select only one event */
+                    numsel = m_seq->select_note_events( tick_s,note_h,tick_s,note_h, 
+                                                        sequence::e_select_one );
+
+                    /* none selected, start selection box */
+                    if ( numsel == 0 )
+                    {
+                        if ( a_ev->button == 1 )
+                            m_selecting = true;
+                    }
+                    else
+                    {
+                        needs_update = true;
+                    }
+                }
+ 
+                
+                if ( m_seq->select_note_events( tick_s, note_h,
+                                                tick_s, note_h, 
+                                                sequence::e_is_selected ))
+                {
+                    if ( a_ev->button == 1 )
+                    {
+                        m_moving_init = true;
+                        needs_update = true;
+
+
+                        /* get the box that selected elements are in */
+                        m_seq->get_selected_box( &tick_s, &note_h, 
+                                &tick_f, &note_l );
+
+
+                        convert_tn_box_to_rect( tick_s, tick_f, note_h, note_l,
+                                &m_selected.x,
+                                &m_selected.y,
+                                &m_selected.width,
+                                &m_selected.height );
+
+                        /* save offset that we get from the snap above */
+                        int adjusted_selected_x = m_selected.x;
+                        snap_x( &adjusted_selected_x );
+                        m_move_snap_offset_x = ( m_selected.x - adjusted_selected_x);
+
+                        /* align selection for drawing */
+                        snap_x( &m_selected.x );
+
+                        m_current_x = m_drop_x = snapped_x;
+                    }
+
+                    /* middle mouse button  */
+                    if ( a_ev->button == 2 ){	
+
+                        /* moving, normal x */
+                        //m_current_x = m_drop_x = norm_x;
+                        //convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
+
+                        m_growing = true;
+
+                        /* get the box that selected elements are in */
+                        m_seq->get_selected_box( &tick_s, &note_h, 
+                                    &tick_f, &note_l );
+
+                        convert_tn_box_to_rect( tick_s, tick_f, note_h, note_l,
+                                &m_selected.x,
+                                &m_selected.y,
+                                &m_selected.width,
+                                &m_selected.height );	
                         
+                    }
+                }
+           }
+       }
 
-                            /* save offset that we get from the snap above */
-                            int adjusted_selected_x = m_selected.x;
-                            snap_x( &adjusted_selected_x );
-                            m_move_snap_offset_x = ( m_selected.x - adjusted_selected_x);
+        /*     right mouse button      */
+        if ( a_ev->button == 3 ){
+            set_adding( true );
+        }
 
-                            /* align selection for drawing */
-                            snap_x( &m_selected.x );
-                            
-                            m_current_x = m_drop_x = snapped_x;
-                        } 
-		}
-    
-		/*     right mouse button      */
-		if ( a_ev->button == 3 ){
-			set_adding( true );
-		}
-		
-		/*     middle mouse button      */
-		if ( a_ev->button == 2 ){	
-			
-			/* moving, normal x */
-			m_current_x = m_drop_x = norm_x;
-			convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
-			
-			/* set to playing */
-			numsel = m_seq->select_note_events( tick_s,   note_h,
-                                                            tick_s+1, note_h );
-			if ( numsel > 0 ){
-				
-				m_growing = true;
-				
-				/* get the box that selected elements are in */
-				m_seq->get_selected_box( &tick_s, &note_h, 
-										 &tick_f, &note_l );
-				
-				convert_tn_box_to_rect( tick_s, tick_f, note_h, note_l,
-										&m_selected.x,
-										&m_selected.y,
-										&m_selected.width,
-										&m_selected.height );	
-			}
-		}
-    }
+   }
 
     /* if they clicked, something changed */
     if ( needs_update ){
-        update_pixmap();
-        queue_draw();
-        m_seqevent_wid->redraw(); 
-        m_seqdata_wid->redraw();
+        redraw();
+        m_seq->set_dirty();
     }
     return true;
-    
+
 }
 
 
-bool
+    bool
 seqroll::on_button_release_event(GdkEventButton* a_ev)
 {
     long tick_s;
@@ -1002,14 +1040,14 @@ seqroll::on_button_release_event(GdkEventButton* a_ev)
     int numsel;
 
     bool needs_update = false;
-    
+
     m_current_x = (int) (a_ev->x + m_scroll_offset_x );
     m_current_y = (int) (a_ev->y + m_scroll_offset_y );
 
     snap_y ( &m_current_y );
 
     if ( m_moving )
-	snap_x( &m_current_x );
+        snap_x( &m_current_x );
 
     int delta_x = m_current_x - m_drop_x;
     int delta_y = m_current_y - m_drop_y;
@@ -1019,55 +1057,65 @@ seqroll::on_button_release_event(GdkEventButton* a_ev)
 
     if ( a_ev->button == 1 ){
 
-	if ( m_selecting ){
-	    
-	    xy_to_rect ( m_drop_x,
-			 m_drop_y,
-			 m_current_x,
-			 m_current_y,
-			 &x, &y,
-			 &w, &h );
-            
-	    convert_xy( x,     y, &tick_s, &note_h );
-	    convert_xy( x+w, y+h, &tick_f, &note_l );
-            
-	    numsel = m_seq->select_note_events( tick_s, note_h, tick_f, note_l );
-            
+        if ( m_selecting ){
+
+            xy_to_rect ( m_drop_x,
+                    m_drop_y,
+                    m_current_x,
+                    m_current_y,
+                    &x, &y,
+                    &w, &h );
+
+            convert_xy( x,     y, &tick_s, &note_h );
+            convert_xy( x+w, y+h, &tick_f, &note_l );
+
+            numsel = m_seq->select_note_events( tick_s, note_h, tick_f, note_l, sequence::e_select );
+
             needs_update = true;
-	}
-        
-	if (  m_moving  ){
+        }
+
+        if (  m_moving  ){
 
             /* adjust for snap */
             delta_x -= m_move_snap_offset_x;
-            
-	    /* convert deltas into screen corridinates */
-	    convert_xy( delta_x, delta_y, &delta_tick, &delta_note );
-	    
-	    /* since delta_note was from delta_y, it will be filpped
-	       ( delta_y[0] = note[127], etc.,so we have to adjust */
-	    delta_note = delta_note - (c_num_keys-1);
-            
+
+            /* convert deltas into screen corridinates */
+            convert_xy( delta_x, delta_y, &delta_tick, &delta_note );
+
+            /* since delta_note was from delta_y, it will be filpped
+               ( delta_y[0] = note[127], etc.,so we have to adjust */
+            delta_note = delta_note - (c_num_keys-1);
+
             m_seq->push_undo();
-	    m_seq->move_selected_notes( delta_tick, delta_note );
+            m_seq->move_selected_notes( delta_tick, delta_note );
             needs_update = true;
-	}
-    }
-    
+        }
+
+   }
+
     if ( a_ev->button == 2 ){	
-	
-	if ( m_growing ){
-	    
-	    /* convert deltas into screen corridinates */
-	    convert_xy( delta_x, delta_y, &delta_tick, &delta_note );
-		m_seq->push_undo();
-	    m_seq->grow_selected( delta_tick );
-        needs_update = true;
-	}
+
+        if ( m_growing ){
+
+            /* convert deltas into screen corridinates */
+            convert_xy( delta_x, delta_y, &delta_tick, &delta_note );
+            m_seq->push_undo();
+
+            if ( a_ev->state & GDK_SHIFT_MASK )
+            {
+                m_seq->stretch_selected( delta_tick );
+            }
+            else
+            {
+                m_seq->grow_selected( delta_tick );
+            }
+
+            needs_update = true;
+        }
     }
 
     if ( a_ev->button == 3 ){	
-	set_adding( false );
+        set_adding( false );
     }
 
     /* turn off */
@@ -1075,15 +1123,16 @@ seqroll::on_button_release_event(GdkEventButton* a_ev)
     m_moving = false;
     m_growing = false;
     m_paste = false;
-	m_moving_init = false;
+    m_moving_init = false;
+    m_painting = false;
 
+    m_seq->unpaint_all();
+ 
     /* if they clicked, something changed */
     if (  needs_update ){
-        
-        update_pixmap();
-        queue_draw();        
-        m_seqevent_wid->redraw(); 
-        m_seqdata_wid->redraw();
+
+        redraw();
+        m_seq->set_dirty();
     }
     return true;
 }
@@ -1120,6 +1169,15 @@ seqroll::on_motion_notify_event(GdkEventMotion* a_ev)
         return true;
 	
     }
+
+    if ( m_painting )
+    {
+        snap_x( &m_current_x );
+        convert_xy( m_current_x, m_current_y, &tick, &note );
+
+        m_seq->add_note( tick, m_note_length - 2, note, true );
+    }
+    
     return false;
 }
 
@@ -1172,7 +1230,7 @@ seqroll::on_focus_in_event(GdkEventFocus*)
 {
     set_flags(Gtk::HAS_FOCUS);
 
-    m_seq->clear_clipboard();
+    //m_seq->clear_clipboard();
 
     return false;
 }
@@ -1193,52 +1251,56 @@ seqroll::on_key_press_event(GdkEventKey* a_p0)
 
     if ( a_p0->type == GDK_KEY_PRESS ){
 
-	if ( a_p0->keyval ==  GDK_Delete ){
+        if ( a_p0->keyval ==  GDK_Delete ){
 
             m_seq->push_undo();
-	    m_seq->remove_selected();
-	    ret = true;
-	}
-        
-	if ( a_p0->state & GDK_CONTROL_MASK ){
-            
-	    /* cut */
-	    if ( a_p0->keyval == GDK_x || a_p0->keyval == GDK_X ){
-		
+            m_seq->mark_selected();
+            m_seq->remove_marked();
+            ret = true;
+        }
+
+        if ( a_p0->state & GDK_CONTROL_MASK ){
+
+            /* cut */
+            if ( a_p0->keyval == GDK_x || a_p0->keyval == GDK_X ){
+
                 m_seq->push_undo();
                 m_seq->copy_selected();
-                m_seq->remove_selected();
-                
+                m_seq->mark_selected();
+                m_seq->remove_marked();
+
                 ret = true;
-	    }
-	    /* copy */
-	    if ( a_p0->keyval == GDK_c || a_p0->keyval == GDK_C ){
-		
+            }
+            /* copy */
+            if ( a_p0->keyval == GDK_c || a_p0->keyval == GDK_C ){
+
                 m_seq->copy_selected();
-                m_seq->unselect();
                 ret = true;
-	    }
-            
+            }
+
             /* paste */
             if ( a_p0->keyval == GDK_v || a_p0->keyval == GDK_V ){
-                
+
                 start_paste();
                 ret = true;
             }
-	}
+
+            /* Undo */
+            if ( a_p0->keyval == GDK_z || a_p0->keyval == GDK_Z ){
+
+                m_seq->pop_undo();
+                ret = true;
+            }
+        }
     }
 
     if ( ret == true ){
-        
-        update_pixmap();
-        queue_draw();        
-        m_seqevent_wid->redraw(); 
-        m_seqdata_wid->redraw();
+        redraw();
+        m_seq->set_dirty();
         return true;
     }
-    else
 
-	return false;
+    return false;
 }
 
 
@@ -1264,3 +1326,22 @@ seqroll::on_size_allocate(Gtk::Allocation& a_r )
     update_sizes(); 
  
 }
+
+bool
+seqroll::on_scroll_event( GdkEventScroll* a_ev )
+{
+	double val = m_vadjust->get_value();
+
+	if ( a_ev->direction == GDK_SCROLL_UP ){
+		val -= m_vadjust->get_step_increment()/6;
+	} else if (  a_ev->direction == GDK_SCROLL_DOWN ){
+		val += m_vadjust->get_step_increment()/6;
+	} else {
+		return true;
+	}
+
+	m_vadjust->clamp_page( val, val + m_vadjust->get_page_size() );
+    return true;
+
+}
+
