@@ -98,6 +98,7 @@ class seqroll : public Gtk::DrawingArea
     bool m_moving_init;
     bool m_growing;
     bool m_adding;
+    bool m_painting;
     bool m_paste;
 
     /* where the dragging started */
@@ -133,6 +134,7 @@ class seqroll : public Gtk::DrawingArea
     bool on_key_press_event(GdkEventKey* a_p0);
     bool on_focus_in_event(GdkEventFocus*);
     bool on_focus_out_event(GdkEventFocus*);
+    bool on_scroll_event( GdkEventScroll* a_ev);
 
     bool on_leave_notify_event	(GdkEventCrossing* a_p0);
     bool on_enter_notify_event	(GdkEventCrossing* a_p0);
