@@ -20,11 +20,14 @@
 #include "sequence.h"
 #include "seqedit.h"
 #include <stdlib.h>
+    
+list < event > sequence::m_list_clipboard;
 
 sequence::sequence( )
 {
  
     m_editing       = false;
+    m_raise         = false;
     m_playing       = false;
     m_was_playing   = false;
     m_recording     = false;
@@ -79,6 +82,7 @@ sequence::pop_undo( void )
         m_list_event = m_list_undo.top();
         m_list_undo.pop();
         verify_and_link();
+        unselect();
     }
 
     unlock();
@@ -415,10 +419,9 @@ sequence::verify_and_link()
 
     lock();
 
-    /* unselect all */
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
-	(*i).unselect();
 	(*i).clear_link();
+    (*i).unmark();
     }
 
     on = m_list_event.begin();
@@ -436,16 +439,16 @@ sequence::verify_and_link()
 	    while ( off != m_list_event.end() ){
 		
 		/* is a off event, == notes, and isnt
-		   selected  */
+		   markeded  */
 		if ( (*off).is_note_off()                  &&
 		     (*off).get_note() == (*on).get_note() && 
-		     ! (*off).is_selected()                  ){
+		     ! (*off).is_marked()                  ){
 
-		    /* link + select */
+		    /* link + mark */
 		    (*on).link( &(*off) );
 		    (*off).link( &(*on) );
-		    (*on).select(  );
-		    (*off).select( );
+		    (*on).mark(  );
+		    (*off).mark( );
 
 		    break;
 		}
@@ -455,26 +458,27 @@ sequence::verify_and_link()
 	on++;
     }
 
-    /* unselect all */
+    /* unmark all */
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
-	(*i).unselect();
+	(*i).unmark();
     }
 
     /* kill those not in range */
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 	
 	/* if our current time stamp is greater then the length */
+
 	if ( (*i).get_timestamp() >= m_length ||
 	     (*i).get_timestamp() < 0            ){
 	    
 	    /* we have to prune it */
-	    (*i).select();
+	    (*i).mark();
 	    if ( (*i).is_linked() )
-		(*i).get_linked()->select();
+		(*i).get_linked()->mark();
 	}
     }
 
-    remove_selected( );
+    remove_marked( );
     unlock();
 }
     
@@ -529,7 +533,7 @@ sequence::link_new( )
 
 
 void 
-sequence::remove_selected( )
+sequence::remove_marked( )
 {
     list<event>::iterator i, t;
 
@@ -538,7 +542,7 @@ sequence::remove_selected( )
     i = m_list_event.begin();
     while( i != m_list_event.end() ){
 	
-	if ((*i).is_selected()){
+	if ((*i).is_marked()){
 	    
 	    /* if its a note off, and that note is currently
 	       playing, send a note off */
@@ -564,6 +568,45 @@ sequence::remove_selected( )
     unlock();
 }
 
+
+
+void 
+sequence::mark_selected( )
+{
+    list<event>::iterator i, t;
+
+    lock();
+
+    i = m_list_event.begin();
+    while( i != m_list_event.end() ){
+	
+	if ((*i).is_selected()){
+
+        (*i).mark();
+    }
+    ++i;
+    }
+    reset_draw_marker();
+
+    unlock();
+}
+
+void 
+sequence::unpaint_all( )
+{
+    list<event>::iterator i;
+
+    lock();
+
+    i = m_list_event.begin();
+    while( i != m_list_event.end() ){
+        (*i).unpaint();
+        i++;
+    }
+    unlock();
+}
+
+
 /* returns the 'box' of the selected items */
 void 
 sequence::get_selected_box( long *a_tick_s, int *a_note_h, 
@@ -699,51 +742,85 @@ sequence::get_num_selected_events( unsigned char a_status,
 
 /* selects events in range..  tick start, note high, tick end
    note low */
-int
+    int
 sequence::select_note_events( long a_tick_s, int a_note_h,
-			      long a_tick_f, int a_note_l )
+        long a_tick_f, int a_note_l, select_action_e a_action)
 {
     int ret=0;
     list<event>::iterator i;
 
     lock();
 
-    
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
-        
-
-        
         if( (*i).is_note_off()                &&
-            (*i).get_timestamp() >= a_tick_s &&
-            (*i).get_note()      <= a_note_h &&
-            (*i).get_note()      >= a_note_l ){
+                (*i).get_timestamp() >= a_tick_s &&
+                (*i).get_note()      <= a_note_h &&
+                (*i).get_note()      >= a_note_l ){
 
-              
-	    if ( (*i).is_linked() ){
 
-		event *ev = (*i).get_linked();
+            if ( (*i).is_linked() ){
 
-		if ( ev->get_timestamp() <= a_tick_f ){
-		    
-		    (*i).select( );
-		    ev->select( );
-		    ret++;
-		}
-	    }
+                event *ev = (*i).get_linked();
+
+                if ( ev->get_timestamp() <= a_tick_f ){
+
+                    if ( a_action == e_select ||
+                         a_action == e_select_one )
+                    {
+                        (*i).select( );
+                        ev->select( );
+                        ret++;
+                        if ( a_action == e_select_one )
+                            break;
+                    }
+                    if ( a_action == e_is_selected )
+                    {
+                        if ( (*i).is_selected())
+                        {
+                            ret = 1;
+                            break;
+                        }
+                    }
+                    if ( a_action == e_would_select )
+                    {
+                        ret = 1;
+                        break;
+                    }
+                }
+            }
         }
-	
-	if ( ! (*i).is_linked() &&
-	     ( (*i).is_note_on() ||
-	       (*i).is_note_off() ) &&
-	     (*i).get_timestamp()  >= a_tick_s - 16 &&
-	     (*i).get_timestamp()  <= a_tick_f && 
-	     (*i).get_note()       <= a_note_h &&
-	     (*i).get_note()       >= a_note_l ) {
 
-	    (*i).select( );
-	    ret++;
-	}
+        if ( ! (*i).is_linked() &&
+                ( (*i).is_note_on() ||
+                  (*i).is_note_off() ) &&
+                (*i).get_timestamp()  >= a_tick_s - 16 &&
+                (*i).get_timestamp()  <= a_tick_f && 
+                (*i).get_note()       <= a_note_h &&
+                (*i).get_note()       >= a_note_l ) {
+
+            if ( a_action == e_select ||
+                 a_action == e_select_one )
+            {
+                (*i).select( );
+                ret++;
+                if ( a_action == e_select_one )
+                    break;
+            }
+            if ( a_action == e_is_selected )
+            {
+                if ( (*i).is_selected())
+                {
+                    ret = 1;
+                    break;
+                }
+            }
+            if ( a_action == e_would_select )
+            {
+                ret = 1;
+                break;
+            }
+        }
     }
 
     unlock();
@@ -751,15 +828,12 @@ sequence::select_note_events( long a_tick_s, int a_note_h,
     return ret;
 }
 
-
-
-
 /* select events in range, returns number
    selected */
 int 
 sequence::select_events( long a_tick_s, long a_tick_f, 
 			 unsigned char a_status, 
-			 unsigned char a_cc )
+			 unsigned char a_cc, select_action_e a_action)
 {
     int ret=0;
     list<event>::iterator i;
@@ -768,28 +842,46 @@ sequence::select_events( long a_tick_s, long a_tick_f,
 
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
-	if( (*i).get_status()    == a_status && 
-	    (*i).get_timestamp() >= a_tick_s &&
-	    (*i).get_timestamp() <= a_tick_f ){
+        if( (*i).get_status()    == a_status && 
+                (*i).get_timestamp() >= a_tick_s &&
+                (*i).get_timestamp() <= a_tick_f ){
 
-	    unsigned char d0,d1;
-	    (*i).get_data( &d0, &d1 );
+            unsigned char d0,d1;
+            (*i).get_data( &d0, &d1 );
 
-	    if ( (a_status == EVENT_CONTROL_CHANGE &&
-		  d0 == a_cc )
-		 || (a_status != EVENT_CONTROL_CHANGE) ){
+            if ( (a_status == EVENT_CONTROL_CHANGE &&
+                        d0 == a_cc )
+                    || (a_status != EVENT_CONTROL_CHANGE) ){
 
-		(*i).select( );
-		ret++;
-	    }
-	}
+
+                if ( a_action == e_select ||
+                     a_action == e_select_one )
+                {
+                    (*i).select( );
+                    ret++;
+                    if ( a_action == e_select_one )
+                        break;
+                }
+                if ( a_action == e_is_selected )
+                {
+                    if ( (*i).is_selected())
+                    {
+                        ret = 1;
+                        break;
+                    }
+                }
+                if ( a_action == e_would_select )
+                {
+                    ret = 1;
+                    break;
+                }
+            }
+        }
     }
-    
-    unlock();
-    
-    return ret;
-}
+        unlock();
 
+        return ret;
+}
 
 
 
@@ -829,16 +921,18 @@ sequence::move_selected_notes( long a_delta_tick, int a_delta_note )
 
     lock();
 
+    mark_selected();
+
     list<event>::iterator i;
 
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
         /* is it being moved ? */
-        if ( (*i).is_selected() ){
+        if ( (*i).is_marked() ){
 
             /* copy event */
             e  = (*i);
-            e.unselect();
+            e.unmark();
 
             if ( (e.get_timestamp() + a_delta_tick) >= 0   &&
                  (e.get_note() + a_delta_note)      >= 0   &&
@@ -846,20 +940,143 @@ sequence::move_selected_notes( long a_delta_tick, int a_delta_note )
 
                 e.set_timestamp( e.get_timestamp() + a_delta_tick );
                 e.set_note( e.get_note() + a_delta_note );
+                e.select();
 
                 add_event( &e );
             }
         }
     }
 
-    remove_selected();
+    remove_marked();
     verify_and_link();
 
     unlock();
 }
 
     
+/* stretch */
+void 
+sequence::stretch_selected( long a_delta_tick )
+{
+    event *e, new_e;
 
+    lock();
+
+    list<event>::iterator i;
+
+    int old_len = 0, new_len = 0;
+    int first_ev = 0x7fffffff;
+    int last_ev = 0x00000000;
+
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+        if ( (*i).is_selected() )
+        {
+            e = &(*i);
+
+            if (e->get_timestamp() < first_ev) {
+                first_ev = e->get_timestamp();
+            }
+            if (e->get_timestamp() > last_ev) {
+                last_ev = e->get_timestamp();
+            }
+        }
+    }
+
+    old_len = last_ev - first_ev;
+    new_len = old_len + a_delta_tick;
+    float ratio = float(new_len)/float(old_len);
+
+    if( new_len > 1) {
+
+        mark_selected();
+        
+        for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+            if ( (*i).is_marked() ){
+
+                e = &(*i);
+
+                /* copy & scale event */
+                new_e = *e;
+                new_e.set_timestamp( long((e->get_timestamp() - first_ev) * ratio) + first_ev );
+               
+                new_e.unmark();
+                
+                add_event( &new_e );
+            }
+        }
+
+        remove_marked();
+        verify_and_link();
+    }
+
+    unlock();
+
+#if 0
+    event *on, *off, new_on, new_off;
+
+    lock();
+
+    list<event>::iterator i;
+
+    int old_len = 0, new_len = 0;
+    int first_ev = 0x7fffffff;
+    int last_ev = 0x00000000;
+
+    for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+        if ( (*i).is_selected() &&
+                (*i).is_note_on() &&
+                (*i).is_linked() ){
+
+            on = &(*i);
+            off = (*i).get_linked();
+
+            if (on->get_timestamp() < first_ev) {
+                first_ev = on->get_timestamp();
+            }
+            if (off->get_timestamp() > last_ev) {
+                last_ev = off->get_timestamp();
+            }
+        }
+    }
+
+    old_len = last_ev - first_ev;
+    new_len = old_len + a_delta_tick;
+    float ratio = float(new_len)/float(old_len);
+
+    if( new_len > 1) {
+
+        mark_selected();
+        
+        for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+            if ( (*i).is_marked() &&
+                    (*i).is_note_on() &&
+                    (*i).is_linked() ){
+
+                on = &(*i);
+                off = (*i).get_linked();
+
+                /* copy & scale event */
+                new_on = *on;
+                new_on.set_timestamp( long((on->get_timestamp() - first_ev) * ratio) + first_ev );
+                new_off = *off;
+                new_off.set_timestamp( long((off->get_timestamp() - first_ev) * ratio) + first_ev );
+               
+                new_on.unmark();
+                new_off.unmark();
+                
+                add_event( &new_on );
+                add_event( &new_off );
+            }
+        }
+
+        remove_marked();
+        verify_and_link();
+    }
+
+    unlock();
+
+#endif
+}
 
 
 /* moves note off event */
@@ -872,9 +1089,11 @@ sequence::grow_selected( long a_delta_tick )
 
     list<event>::iterator i;
 
+    mark_selected();
+    
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
-        if ( (*i).is_selected() &&
+        if ( (*i).is_marked() &&
                 (*i).is_note_on() &&
                 (*i).is_linked() ){
 
@@ -885,23 +1104,21 @@ sequence::grow_selected( long a_delta_tick )
                 off->get_timestamp() - 
                 on->get_timestamp() +
                 a_delta_tick;
+            on->unmark();
 
             if ( length < 1 )
                 length = 1;
 
-            /* unselect it */
-            on->unselect();
-
             /* copy event */
             e  = *off;
-            e.unselect();
+            e.unmark();
 
             e.set_timestamp( on->get_timestamp() + length );
             add_event( &e );
         }
     }
 
-    remove_selected();
+    remove_marked();
     verify_and_link();
     
     unlock();
@@ -976,10 +1193,6 @@ sequence::decrement_selected(unsigned char a_status, unsigned char a_control )
 
 
 
-void sequence::clear_clipboard( void )
-{
-    m_list_clipboard.clear( );
-}
 
 void
 sequence::copy_selected( void )
@@ -1014,26 +1227,27 @@ sequence::paste_selected( long a_tick, int a_note )
     int highest_note = 0;
 
     lock();
+    list<event> clipboard = m_list_clipboard;
 
-    for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
+    for ( i = clipboard.begin(); i != clipboard.end(); i++ ){
 	(*i).set_timestamp((*i).get_timestamp() + a_tick );
     }
 
-    if ((*m_list_clipboard.begin()).is_note_on() ||
-	(*m_list_clipboard.begin()).is_note_off() ){
+    if ((*clipboard.begin()).is_note_on() ||
+	(*clipboard.begin()).is_note_off() ){
 	
-	for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ )
+	for ( i = clipboard.begin(); i != clipboard.end(); i++ )
 	    if ( (*i).get_note( ) > highest_note ) highest_note = (*i).get_note();
 
        
 
-	for ( i = m_list_clipboard.begin(); i != m_list_clipboard.end(); i++ ){
+	for ( i = clipboard.begin(); i != clipboard.end(); i++ ){
 
 	    (*i).set_note( (*i).get_note( ) - (highest_note - a_note) );
 	}
     }
 
-    m_list_event.merge( m_list_clipboard );
+    m_list_event.merge( clipboard );
     m_list_event.sort();
 
     verify_and_link();
@@ -1050,76 +1264,98 @@ void
 sequence::change_event_data_range( long a_tick_s, long a_tick_f,
 				   unsigned char a_status,
 				   unsigned char a_cc,
-				   unsigned char a_data_s,
-				   unsigned char a_data_f )
+				   int a_data_s,
+				   int a_data_f )
 {
     lock();
 
     unsigned char d0, d1;
     list<event>::iterator i;
 
+    /* change only selected events, if any */
+    bool have_selection = false;
+    if( get_num_selected_events(a_status, a_cc) )
+        have_selection = true;
+
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
-	/* initially false */
-	bool set = false;	
-	(*i).get_data( &d0, &d1 );
-	
-	/* correct status and not CC */
-	if ( a_status != EVENT_CONTROL_CHANGE &&
-	     (*i).get_status() == a_status )
-	    set = true;
-	
-	/* correct status and correct cc */
-	if ( a_status == EVENT_CONTROL_CHANGE &&
-	     (*i).get_status() == a_status &&
-	     d0 == a_cc )
-	    set = true;
-	
-	/* in range? */
-	if ( !((*i).get_timestamp() >= a_tick_s &&
-	       (*i).get_timestamp() <= a_tick_f ))
-	    set = false;
-	
-	if ( set ){
-	    
-	    float weight;
-	    
-	    /* no divide by 0 */
-	    if( a_tick_f == a_tick_s )
-		a_tick_f = a_tick_s + 1;
-	    
-	    /* ratio of v1 to v2 */
-	    weight = 
-		(float)( (*i).get_timestamp() - a_tick_s ) /
-		(float)( a_tick_f - a_tick_s ); 
-	    
-	    int newdata = (int) 
-		((weight         * (float) a_data_f ) +
-		 ((1.0f - weight) * (float) a_data_s ));
-	    
-	    if ( a_status == EVENT_NOTE_ON )
-		d1 = newdata;
+        /* initially false */
+        bool set = false;	
+        (*i).get_data( &d0, &d1 );
 
-	    if ( a_status == EVENT_NOTE_OFF )
-		d1 = newdata;
+        /* correct status and not CC */
+        if ( a_status != EVENT_CONTROL_CHANGE &&
+                (*i).get_status() == a_status )
+            set = true;
 
-	    if ( a_status == EVENT_AFTERTOUCH )
-		d1 = newdata;
+        /* correct status and correct cc */
+        if ( a_status == EVENT_CONTROL_CHANGE &&
+                (*i).get_status() == a_status &&
+                d0 == a_cc )
+            set = true;
 
-	    if ( a_status == EVENT_CONTROL_CHANGE )
-		d1 = newdata;
+        /* in range? */
+        if ( !((*i).get_timestamp() >= a_tick_s &&
+                    (*i).get_timestamp() <= a_tick_f ))
+            set = false;
 
-	    if ( a_status == EVENT_PROGRAM_CHANGE )
-		d0 = newdata; /* d0 == new patch */
+        /* in selection? */
+        if ( have_selection && (!(*i).is_selected()) )
+            set = false;
 
-	    if ( a_status == EVENT_CHANNEL_PRESSURE )
-		d0 = newdata; /* d0 == pressure */
+        if ( set ){
 
-	    if ( a_status == EVENT_PITCH_WHEEL )
-		d1 = newdata;
+            float weight;
 
-	    (*i).set_data( d0, d1 );
-	}	    
+            /* no divide by 0 */
+            if( a_tick_f == a_tick_s )
+                a_tick_f = a_tick_s + 1;
+
+            /* ratio of v1 to v2 */
+            /*
+               weight =
+               (float)( (*i).get_timestamp() - a_tick_s ) /
+               (float)( a_tick_f - a_tick_s );
+
+               int newdata = (int)
+               ((weight         * (float) a_data_f ) +
+               ((1.0f - weight) * (float) a_data_s ));
+               */
+
+            int tick = (*i).get_timestamp();
+
+            //printf("ticks: %d %d %d\n", a_tick_s, tick, a_tick_f);
+            //printf("datas: %d %d\n", a_data_s, a_data_f);
+
+            int newdata = ((tick-a_tick_s)*a_data_f + (a_tick_f-tick)*a_data_s)
+                /(a_tick_f - a_tick_s);
+
+            if ( newdata < 0 ) newdata = 0;
+            if ( newdata > 127 ) newdata = 127;
+
+            if ( a_status == EVENT_NOTE_ON )
+                d1 = newdata;
+
+            if ( a_status == EVENT_NOTE_OFF )
+                d1 = newdata;
+
+            if ( a_status == EVENT_AFTERTOUCH )
+                d1 = newdata;
+
+            if ( a_status == EVENT_CONTROL_CHANGE )
+                d1 = newdata;
+
+            if ( a_status == EVENT_PROGRAM_CHANGE )
+                d0 = newdata; /* d0 == new patch */
+
+            if ( a_status == EVENT_CHANNEL_PRESSURE )
+                d0 = newdata; /* d0 == pressure */
+
+            if ( a_status == EVENT_PITCH_WHEEL )
+                d1 = newdata;
+
+            (*i).set_data( d0, d1 );
+        }	    
     }
 
     unlock();
@@ -1129,28 +1365,58 @@ sequence::change_event_data_range( long a_tick_s, long a_tick_f,
 
 
 void 
-sequence::add_note( long a_tick, long a_length, int a_note )
+sequence::add_note( long a_tick, long a_length, int a_note, bool a_paint)
 {
 
     lock();
 
+    event e;
+    
     if ( a_tick >= 0 && 
-	 a_note >= 0 &&
-	 a_note < c_num_keys ){
+         a_note >= 0 &&
+         a_note < c_num_keys ){
 
-	event e;
+        /* if we care about the painted, run though 
+         * our events, delete the painted ones that
+         * overlap the one we want to add */
+        if ( a_paint )
+        {
+            list<event>::iterator i,t;
+            for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
-	e.set_status( EVENT_NOTE_ON );
-	e.set_data( a_note, 100 );
-	e.set_timestamp( a_tick );
+                if ( (*i).is_painted() &&
+                     (*i).is_note_on() &&
+                     (*i).get_timestamp() == a_tick )
+                {
+                    (*i).mark();
 
-	add_event( &e );
+                    if ( (*i).is_linked())
+                    {
+                        (*i).get_linked()->mark();
+                    }
 
-	e.set_status( EVENT_NOTE_OFF );
-	e.set_data( a_note, 100 );
-	e.set_timestamp( a_tick + a_length );
+                    set_dirty();
+                }
+            }
 
-	add_event( &e );
+            remove_marked();
+        }                        
+        
+
+        if ( a_paint )
+            e.paint();
+        
+        e.set_status( EVENT_NOTE_ON );
+        e.set_data( a_note, 100 );
+        e.set_timestamp( a_tick );
+
+        add_event( &e );
+
+        e.set_status( EVENT_NOTE_OFF );
+        e.set_data( a_note, 100 );
+        e.set_timestamp( a_tick + a_length );
+
+        add_event( &e );
     }
 
     verify_and_link();
@@ -1162,19 +1428,50 @@ void
 sequence::add_event( long a_tick, 
 		     unsigned char a_status,
 		     unsigned char a_d0,
-		     unsigned char a_d1 )
+		     unsigned char a_d1,
+             bool a_paint)
 {
     lock();
 
     if ( a_tick >= 0 ){
-	
-	event e;
 
-	e.set_status( a_status );
-	e.set_data( a_d0, a_d1 );
-	e.set_timestamp( a_tick );
+        event e;
 
-	add_event( &e );
+        /* if we care about the painted, run though 
+         * our events, delete the painted ones that
+         * overlap the one we want to add */
+        if ( a_paint )
+        {
+            list<event>::iterator i,t;
+            for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
+
+                if ( (*i).is_painted() &&
+                     (*i).get_timestamp() == a_tick )
+                {
+                    (*i).mark();
+
+                    if ( (*i).is_linked())
+                    {
+                        (*i).get_linked()->mark();
+                    }
+
+                    set_dirty();
+                }
+            }
+
+            remove_marked();
+        }                        
+        
+
+        if ( a_paint )
+            e.paint();
+ 
+        
+        e.set_status( a_status );
+        e.set_data( a_d0, a_d1 );
+        e.set_timestamp( a_tick );
+
+        add_event( &e );
     }
     verify_and_link();
 
@@ -2400,7 +2697,7 @@ void
 sequence::set_length( long a_len, bool a_adjust_triggers )
 {
     lock();
- 
+
     bool was_playing = get_playing();
 
     /* turn everything off */
@@ -2688,11 +2985,12 @@ sequence::transpose_notes( int a_steps, int a_scale )
 {
     event e;
 
-    //unselect( );
     list<event> transposed_events;
 
     lock();
 
+    mark_selected();
+    
     list<event>::iterator i;
 
     const int *transpose_table = NULL;
@@ -2710,9 +3008,10 @@ sequence::transpose_notes( int a_steps, int a_scale )
 	/* is it being moved ? */
 	if ( ((*i).get_status() ==  EVENT_NOTE_ON ||
               (*i).get_status() ==  EVENT_NOTE_OFF) &&
-             (*i).is_selected() ){
+             (*i).is_marked() ){
 
             e = (*i);
+            e.unmark();
 
             int  note = e.get_note();
 
@@ -2732,11 +3031,10 @@ sequence::transpose_notes( int a_steps, int a_scale )
    
             transposed_events.push_front(e);
 
-            (*i).select();
 	}
     }
 
-    remove_selected();
+    remove_marked();
     transposed_events.sort();
     m_list_event.merge( transposed_events);
     
@@ -2762,9 +3060,10 @@ sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
     unsigned char d0, d1;
     list<event>::iterator i;
 
-    //unselect( );
     list<event> quantized_events;
 
+    mark_selected();
+    
     for ( i = m_list_event.begin(); i != m_list_event.end(); i++ ){
 
         /* initially false */
@@ -2782,7 +3081,7 @@ sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
 	     d0 == a_cc )
 	    set = true;
 
-        if( !(*i).is_selected() )
+        if( !(*i).is_marked() )
             set = false;
 	
         if ( set ){
@@ -2790,6 +3089,7 @@ sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
             /* copy event */
 	    e = (*i);
             (*i).select();
+            e.unmark();
 
             long timestamp = e.get_timestamp();
             long timestamp_remander = (timestamp % a_snap_tick);
@@ -2808,6 +3108,7 @@ sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
             if ( (*i).is_linked() && a_linked ){
                 
                 f = *(*i).get_linked();
+                f.unmark();
                 (*i).get_linked()->select();
                 
                 f.set_timestamp( f.get_timestamp() + timestamp_delta );
@@ -2817,7 +3118,7 @@ sequence::quanize_events( unsigned char a_status, unsigned char a_cc,
 
     }
 
-    remove_selected();
+    remove_marked();
     quantized_events.sort();
     m_list_event.merge(quantized_events);
     verify_and_link();
@@ -3028,4 +3329,4 @@ sequence::fill_list( list<char> *a_list, int a_pos )
 //     addLongList( a_list, c_triggersmidibus );
 
 
-   
+
